import pix4dsrs as srs
from collections import UserDict
from catchcrs.utils.srs import SRSUtils
srs.init(True)


class CRS():
    code = -1
    name = ""
    type = ""
    wktString = ""
    axes = []
    extents = None

    def __init__(self, srs_catalog_item):
        if isinstance(srs_catalog_item, srs.SrsCatalogItem):
            spat_ref = SRSUtils.get_spatial_reference(srs_catalog_item.authority, srs_catalog_item.code)

            self.code = int(srs_catalog_item.code)
            self.name = srs_catalog_item.name
            self.wktString =  srs.wkt_from_user_input(SRSUtils.make_string_identifier(srs_catalog_item.authority.name, srs_catalog_item.code), srs.WktConvention.WKT2_2019, multiline=False)
            self.axes = SRSUtils.axes_to_dict(spat_ref.axes)

            if srs_catalog_item.areas_of_use:
                if not SRSUtils.area_of_use_worldwide(srs_catalog_item.areas_of_use):
                    self.extents = SRSUtils.area_of_use_to_dict(srs_catalog_item.areas_of_use)

    def to_dict(self):
        tmp = {
            "code" : self.code,
            "name" : self.name,
            "type" : self.type,
            "axes" : self.axes
        }

        if self.extents:
            tmp["extents"] = self.extents
        
        return tmp


class GeographicCrs(CRS):
    def __init__(self, srs_catalog_item):
        if isinstance(srs_catalog_item, srs.SrsCatalogItem):
            self.type = "GEOGCS"
            super().__init__(srs_catalog_item)


class VerticalCrs(CRS):
    geoids = []
    
    def __init__(self, catalog, srs_catalog_item):
        if isinstance(srs_catalog_item, srs.SrsCatalogItem):
            self.geoids = SRSUtils.get_geoids(catalog, srs_catalog_item.authority, srs_catalog_item.code)
            self.type = "VERTCS"

            super().__init__(srs_catalog_item)

    def to_dict(self):
        inh = super().to_dict()

        if self.geoids and self.geoids is not None:
            inh["geoids"] = self.geoids

        return inh


class ProjectedCrs(CRS):
    def __init__(self, srs_catalog_item):
        if isinstance(srs_catalog_item, srs.SrsCatalogItem):
            self.type = "PROJCS"

            super().__init__(srs_catalog_item)

    def to_dict(self):
        return super().to_dict()
