import pix4dsrs as srs

class SRSUtils:
    
    @staticmethod
    def make_string_identifier(authority, code):
        if isinstance(authority, str):
            return authority + ":" + str(code)
        if isinstance(authority, srs._pix4dsrs.Authority):
            return authority.name + ":" + str(code)

    @staticmethod
    def get_spatial_reference(authority, code):
        return srs.SpatialReference(srs.wkt_from_user_input(SRSUtils.make_string_identifier(authority, code)))

    @staticmethod
    def string_to_authority(authority):
        if isinstance(authority, str):
            return {
                "epsg": srs.Authority.EPSG,
                "EPSG": srs.Authority.EPSG,

                "esri": srs.Authority.ESRI,
                "ESRI": srs.Authority.ESRI,

                "ignf": srs.Authority.IGNF,
                "IGNF": srs.Authority.IGNF,
                
                "nkg": srs.Authority.NKG,
                "NKG": srs.Authority.NKG,

                "ogc": srs.Authority.OGC,
                "OGC": srs.Authority.OGC,

                "proj": srs.Authority.PROJ,
                "PROJ": srs.Authority.PROJ
            }[authority, None]
        elif isinstance(authority, srs.Authority):
            return authority
        else:
            return None

    @staticmethod
    def area_of_use_to_dict(area_of_use):
        if isinstance(area_of_use, srs._pix4dsrs.AreaOfUse):
            return [ area_of_use.east, area_of_use.north, area_of_use.south, area_of_use.west ]
        if isinstance(area_of_use, list):
            if len(area_of_use) == 1:
                return [ area_of_use[0].east, area_of_use[0].north, area_of_use[0].south, area_of_use[0].west ]
        
        return None

    @staticmethod
    def area_of_use_worldwide(area_of_use):
        if isinstance(area_of_use, srs._pix4dsrs.AreaOfUse):
            if area_of_use.east == 180.0 and area_of_use.north == 90.0 and area_of_use.south == -90.0 and area_of_use.west == -180.0:
                return True
            else :
                return False
        if isinstance(area_of_use, list):
            if len(area_of_use) == 1:
                if area_of_use[0].east == 180.0 and area_of_use[0].north == 90.0 and area_of_use[0].south == -90.0 and area_of_use[0].west == -180.0:
                    return True
                else :
                    return False

        return None

    @staticmethod
    def axis_to_dict(axis):
        if not isinstance(axis, srs._pix4dsrs.Axis):
            return None

        return {
            "label" : axis.name,
            "unit" : axis.unit_name
        }

    @staticmethod
    def axes_to_dict(axes):
        if not isinstance(axes, list):
            return None

        out = []
        for axis in axes:
            out.append(SRSUtils.axis_to_dict(axis))

        return out

    @staticmethod
    def get_geoids(catalog, authority, code):
        if not isinstance(catalog, srs._pix4dsrs.SrsCatalog):
            return None

        if isinstance(code, int):
            code = str(code)

        if not isinstance(code, str):
            return None
        
        if isinstance(authority, str):
            authority = SRSUtils.string_to_authority(authority)

        if isinstance(authority, srs.Authority):
            return catalog.geoid_models(authority, code)
        else:
            return None
