import pix4dsrs as srs
import getopt
import sys
import os
from collections import UserDict

srs.init(True)

from catchcrs.utils.file import FileUtils
from catchcrs.crs import VerticalCrs, ProjectedCrs, GeographicCrs

VERTICAL_CRS_FILE_NAME = "EPSGVerticalCoordinateReferenceSystems.json"
PROJECTED_CRS_FILE_NAME = "EPSGProjectedCoordinateReferenceSystems.json"
GEOGRAPHIC_CRS_FILE_NAME = "EPSGGeographicCoordinateReferenceSystems.json"

"""
Generates the CRS files
"""

def get_vertical_crss():
    catalog = srs.SrsCatalog()
    catalog_vertical = catalog.find_srss([srs.SpatialReferenceType.VERTICAL])

    out = []
    for item in catalog_vertical:
        out.append(VerticalCrs(catalog, item).to_dict())

    return out


def get_projected_crss():
    catalog = srs.SrsCatalog()
    catalog_projected = catalog.find_srss([srs.SpatialReferenceType.PROJECTED])

    out = []
    for index, item in enumerate(catalog_projected):
        if index % int(len(catalog_projected) / 10) == 0:
            print("         " + str(int(index / int(len(catalog_projected) / 10)) * 10) + "%")
        
        out.append(ProjectedCrs(item).to_dict())

    return out


def get_geographic_crss():
    catalog = srs.SrsCatalog()
    catalog_geographic = catalog.find_srss([srs.SpatialReferenceType.GEOGRAPHIC_2D])

    out = []
    for item in catalog_geographic:
        out.append(GeographicCrs(item).to_dict())
    
    return out

if __name__ == '__main__':
    output_dir = ""
    try:
        opts, args = getopt.getopt(sys.argv[1:], "o:", ["out="])
    except getopt.GetoptError:
        print('Invalid input')
        sys.exit(2)
    for opt, arg in opts:
        # NOTE : getopt doesn't force required arguments
        # i.e. even if we require v with v:, getopt won't throw GetoptError

        if opt == "--out" or opt == "-o":
            output_dir = arg[1:]

    #If the supplied path isn't valid, try 
    if not os.path.exists(output_dir):
        output_dir = os.getcwd() + output_dir

    print("Output directory : " + output_dir)

    if not os.path.exists(output_dir):
        raise Exception("Path doesn't exist!")

    if not os.path.isdir(output_dir):
        raise Exception("Output is not a directory!")

    if len(os.listdir(output_dir)) > 0 :
        print("Output directory is not empty ! Do you with to continue ? (Y/n)")
        
        inp = input()
        inp = str(inp).lower()

        if inp == "y" or inp == "yes":
            print("Continuing even though folder is not empty, at the user's command")
        else:
            raise Exception("Output directory is not empty and user refused to continue")
    
    #ALL CHECKS SHOULD HAVE BEEN PASSED
    if not output_dir[-1] == "/":
        output_dir += "/"

    output_vertical_crs = output_dir + VERTICAL_CRS_FILE_NAME

    print("Generating file : 1/3 (vertical)")
    vert = get_vertical_crss()
    FileUtils.write_file(output_vertical_crs, vert)
    print("Writing file    : 1/3 (vertical)")

    output_geographic_crs = output_dir + GEOGRAPHIC_CRS_FILE_NAME

    print("Generating file : 2/3 (geographic)")
    geog = get_geographic_crss()
    FileUtils.write_file(output_geographic_crs, geog)
    print("Writing file    : 2/3 (geographic)")


    print("Generating file : 3/3 (projected)")
    output_projected_crs = output_dir + PROJECTED_CRS_FILE_NAME

    proj = get_projected_crss()
    FileUtils.write_file(output_projected_crs, proj)
    print("Writing file    : 3/3 (projected)")
    print("Done!")
