import os
import getopt
import sys
import pix4dsrs as srs
from pix4dsrs import ext
import numpy as np

# Using pix4d srs from srs 4.4.0 (https://artifactory.ci.pix4d.com/ui/repos/tree/General/pix4d-pypi-local/pix4dsrs/4.4.0)

if "PROJLIB" in os.environ:
    srs.set_proj_lib_paths([os.environ["PROJLIB"]])
else:
    srs.init(False)


def get_site_localization_wkt2(srs_in, reference_coord, site_coord, name):
    wkt2 = ext.make_site_calibration_wkt2(srs_in, reference_coord, srs.LengthUnitType.Metre, site_coord, name)
    print(wkt2)
    return wkt2

def save_wkt2(wkt, name):
    text_file = open("./"+name+".wkt", "w")
    text_file.write(wkt)
    text_file.close
    print("file saved : ", "./"+name+".wkt")

def compute_rms_and_max_residuals(source_crs, source_coords, site_crs, site_coord):
    converter = srs.CoordinatesConverter(from_srs=source_crs, to_srs=site_crs)

    residuals = 0.0
    max_residuals = 0.0
    for i, site in enumerate(site_coord):
        r = converter.convert(source_coords[i])
        res = np.linalg.norm(r - site)**2
        max_residuals = max(res, max_residuals)
        residuals += res

    residuals /= len(site_coord)
    residuals = np.sqrt(residuals)
    return {residuals, np.sqrt(max_residuals)}
# compute RMs and ma

if __name__ == '__main__':
    # Global point CRS
    srs_in = srs.SpatialReference(srs.wkt_from_user_input("EPSG:9057"))

    # Global points
    v_ref = [[38.907358065060, -77.036495018355, -1.299786383286],
                [38.907358063658, -77.035918577959, -1.299790622666],
                [38.907808458536, -77.035918574344, -1.299777153879]]

    # Local points
    v_site = [[5000.0, 5000.0, 0.0],
                [5035.355339, 5035.355339, 0.0],
                [5000.0, 5070.71067, 0.0]]

    name = "site_localization"
    try:
        opts, args = getopt.getopt(sys.argv[1:], "n:", ["name="])
    except getopt.GetoptError:
        print('Invalid input')
        sys.exit(2)
    for opt, arg in opts:
        # NOTE : getopt doesn't force required arguments
        # i.e. even if we require v with v:, getopt won't throw GetoptError

        if opt == "--name" or opt == "-n":
            name = arg[1:]

    wkt2 = get_site_localization_wkt2(srs_in, v_ref, v_site, name)
    save_wkt2(wkt2, name)
    site_srs = srs.SpatialReference(wkt2)
    rms, max_res = compute_rms_and_max_residuals(srs_in, v_ref, site_srs, v_site)
    print("RMS:", rms, "Max residuals:", max_res)
